// GraphicDebugInfo.cpp : GraphicDebugInfo Dialog box.  This dialog box displays a 
//	map of the memory used for the graphics surfaces of the RGB processor.  
//

#include "stdafx.h"
#include "tulsa sample app.h"
#include "GraphicDebugInfo.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define MEM_SIZE	 0x800000

int TextBoxIDs[] = {IDC_ITEM1, IDC_ITEM2, IDC_ITEM3, IDC_ITEM4, IDC_ITEM5, IDC_ITEM6, IDC_ITEM7};
int KeyIDs[] = {IDC_BOX1, IDC_BOX2, IDC_BOX3, IDC_BOX4, IDC_BOX5, IDC_BOX6, IDC_BOX7};
COLORREF ItemColor[] = {RGB(0xc0, 0x00, 0x00),
						RGB(0x00, 0xc0, 0x00),
						RGB(0x00, 0x00, 0xc0),
						RGB(0xc0, 0xc0, 0x00),
						RGB(0x00, 0xc0, 0xc0),
						RGB(0xc0, 0x00, 0xc0),
						RGB(0xff, 0xff, 0xff)};

COLORREF CmdBarColors[] = {RGB(0x00, 0x00, 0x00),
						   RGB(0x00, 0xff, 0xff),
						   RGB(0xff, 0xff, 0x00),
						   RGB(0x00, 0xff, 0x00)};

#define STATUS_UPDATE_PERIOD	100

/////////////////////////////////////////////////////////////////////////////
// CGraphicDebugInfo dialog


CGraphicDebugInfo::CGraphicDebugInfo(CWnd* pParent /*=NULL*/)
	: CDialog(CGraphicDebugInfo::IDD, pParent)
{
	//{{AFX_DATA_INIT(CGraphicDebugInfo)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	m_Items = 0;
	m_pVScrollPos = NULL;
	m_TimerID = 0;
}


void CGraphicDebugInfo::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CGraphicDebugInfo)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CGraphicDebugInfo, CDialog)
	//{{AFX_MSG_MAP(CGraphicDebugInfo)
	ON_WM_PAINT()
	ON_WM_DESTROY()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGraphicDebugInfo message handlers

BOOL CGraphicDebugInfo::OnInitDialog() 
{
	char str[200];
	LPTULSASURFACEEXT lpTulsaSurfaceExt;

	CDialog::OnInitDialog();

	HRESULT result = CoCreateInstance(CLSID_STBTDDExtension, NULL, 
		CLSCTX_INPROC, IID_ISTBTDDSurfaceExt, (LPVOID *)&lpTulsaSurfaceExt);
	if (result == DD_OK) {
		lpTulsaSurfaceExt->AssignSurface(m_pSurface);

		lpTulsaSurfaceExt->GetSurfaceInfo((DEBUG_SURFACE_INFO*)&m_DebugInfo, MAX_DEBUG_ITEMS, &m_Items);

		lpTulsaSurfaceExt->Release();
	}

	for (int i = 0; i < (int)m_Items; i++) {
		char *szType;
		char *szOverlayType = "";
		if (m_DebugInfo[i].m_bPrimary) {
			szType = "Primary";
		} else {
			szType = "Overlay";

			if (m_DebugInfo[i].m_SurfaceDescExt.dwFlags & SDE_SCROLL_VERT) {
				szOverlayType = "Vert Scroll,";
			} else if (m_DebugInfo[i].m_SurfaceDescExt.dwFlags & SDE_SCROLL_HORZ) {
				szOverlayType = "Horz Scroll,";
			}
		}

		DWORD StartAddress = min(m_DebugInfo[i].m_dwOddBase, m_DebugInfo[i].m_dwEvenBase);
		DWORD EndAddress = StartAddress + m_DebugInfo[i].m_SurfaceDesc.dwHeight*m_DebugInfo[i].m_SurfaceDesc.lPitch;
		sprintf(str, "Surface Type = %s,  %s  Address Range:[%x:%x], Size= %x, SurfCmd#= %d", szType, szOverlayType, StartAddress, EndAddress, EndAddress - StartAddress, m_DebugInfo[i].m_CommandCount);
		SetDlgItemText(TextBoxIDs[i], str);
	}

	m_TimerID = SetTimer(0, STATUS_UPDATE_PERIOD, NULL);


	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CGraphicDebugInfo::GetCntlRect(int ID, CRect &rect)
{
	CRect rectDlg;
	CRect rectTemp;
	CRect rectCntl;

	GetWindowRect(&rectDlg);

	CWnd *pWnd = (CWnd*)GetDlgItem(ID);
	pWnd->GetWindowRect(&rectCntl);
	
	rectCntl -= rectDlg.TopLeft();

	rectCntl.top -= GetSystemMetrics(SM_CYCAPTION) + GetSystemMetrics(SM_CYDLGFRAME);
	rectCntl.bottom -= GetSystemMetrics(SM_CYCAPTION) + GetSystemMetrics(SM_CYDLGFRAME);
	rectCntl.left -= GetSystemMetrics(SM_CXDLGFRAME);
	rectCntl.right -= GetSystemMetrics(SM_CXDLGFRAME);

	pWnd->GetClientRect(&rectTemp);
	rect = rectCntl;
}

void CGraphicDebugInfo::OnPaint() 
{
	CPaintDC dc(this); // device context for painting

	CRect rectMemMap;
	CRect rectMemItem;
	CRect rectItemKey;

	GetCntlRect(IDC_MEM_MAP, rectMemMap);

	int Width = rectMemMap.Width();
	int Height = rectMemMap.Height();
	rectMemItem = rectMemMap;

	for (int i = 0; i < (int)m_Items; i++) {
		DWORD StartAddress = min(m_DebugInfo[i].m_dwOddBase, m_DebugInfo[i].m_dwEvenBase);
		DWORD EndAddress = StartAddress + m_DebugInfo[i].m_SurfaceDesc.dwHeight*m_DebugInfo[i].m_SurfaceDesc.lPitch;

		rectMemItem.left = rectMemMap.left + Width*StartAddress/MEM_SIZE;
		rectMemItem.right = rectMemMap.left + Width*EndAddress/MEM_SIZE;

		CBrush brush(ItemColor[i]);

		rectMemItem.bottom -= Height/6;
		dc.FillRect(&rectMemItem, &brush);
		rectMemItem.bottom += Height/6;

		GetCntlRect(KeyIDs[i], rectItemKey);

		dc.FillRect(&rectItemKey, &brush);

		int CmdBarColorIndex = ((m_DebugInfo[i].m_CommandCount % 3) + 1);
		CBrush brush2(CmdBarColors[CmdBarColorIndex]);

		rectMemItem.top += 5*Height/6;
		dc.FillRect(&rectMemItem, &brush2);
		rectMemItem.top -= 5*Height/6;

//fix - temp
#if 1 
		if (m_DebugInfo[i].m_SurfaceDescExt.dwFlags & SDE_SCROLL_VERT) {
			if (m_pVScrollPos != NULL) {
				int ScrollMarker = *m_pVScrollPos*0xc00*1;
				rectMemItem.left += Width*ScrollMarker/MEM_SIZE;

				CPen pen(PS_DASH, 1, RGB(0, 0, 0));
				dc.MoveTo(rectMemItem.left, rectMemItem.top);
				dc.LineTo(rectMemItem.left, rectMemItem.bottom);
			}
		}
#endif
	}
}

void CGraphicDebugInfo::OnDestroy() 
{
	CDialog::OnDestroy();
	
	if (m_TimerID) {
		KillTimer(m_TimerID);
	}
}

void CGraphicDebugInfo::OnTimer(UINT nIDEvent) 
{
	InvalidateRect(NULL, FALSE);
	
	CDialog::OnTimer(nIDEvent);
}
